function [SR, report] = llrMultichannel(LRImages, model, optimParams, varargin)
    
    if nargin < 3
        optimParams = getLLROptimizationParams;
    end
    
    % Get ground truth image for evaluation purposes if available.
    if nargin > 3
        gt = varargin{1};
        if ~iscell(gt)
            for channelIdx = 1:size(gt, 3)
                I{channelIdx} = gt(:,:,channelIdx);
            end
            gt = I;
        end
    end
    
    % Initialization for all channels.
    numChannels = length(LRImages);
    numFrames = size(LRImages{1}, 3);
    for channelIdx = 1:numChannels  
        I_guidance = LRImages{channelIdx};
        lrDim{channelIdx} = size(I_guidance(:,:,1));
        
        % Initialize the system matrices for all frames associated with the
        % current channel.
        for frameIdx = 1:numFrames
            I = LRImages{channelIdx};
            if isscalar(model.psfWidth)
                % Each channel has the same PSF model.
                W{frameIdx} = composeSystemMatrix(size(I(:,:,frameIdx)), model.magFactor, model.psfWidth, model.motionParams{frameIdx});
                Wt{frameIdx} = W{frameIdx}';
            else
                % Each channel has a different PSF model. Build the system 
                % matrix for each channel separately.
                W{channelIdx,frameIdx} = composeSystemMatrix(size(I(:,:,frameIdx)), model.magFactor, model.psfWidth{channelIdx}, model.motionParams{frameIdx});
                Wt{channelIdx,frameIdx} = W{channelIdx,frameIdx}';
            end
            
            % Initialize the confidence weights of the observation
            % model.
            model.confidence{channelIdx,frameIdx} = ones(numel(I(:,:,frameIdx)), 1);
        end
        
        % Initialize the super-resolved channel for the user-defined
        % initial guess.
        SR{channelIdx} = imageToVector(model.SR{channelIdx});
        
        % Initialize the confidence weights of the LLR model.
        for m = 1:numChannels
            llrPrior.confidence{channelIdx,m} = ones(model.magFactor^2 * numel(I(:,:,1)), 1);
        end
    end
            
    scgOptions = setupSCGOptions(optimParams);    
    isConverged = false;
    iter = 1;
    while iter <= optimParams.maxMMIter && ~isConverged
        
        if nargout > 1
            if exist('gt', 'var')
                % Evaluate super-resolved data for the current iteration if
                % ground truth data is provided.
                for k = 1:numChannels
                    report.psnrMeasures(k, iter) = psnr(vectorToImage(SR{k}, model.magFactor * lrDim{k}), gt{k});
                    report.ssimMeasures(k, iter) = ssim(vectorToImage(SR{k}, model.magFactor * lrDim{k}), gt{k});
                end
            end
            if iter > 1
                report.deltaSR(iter-1) = sum( abs(deltaSR(:)) );
                report.sigmaNoise(:,iter-1) = sigmaNoise;
                report.sigmaBTV(:,iter-1) = sigmaBTV;
                if optimParams.adaptiveLLR
                    report.sigmaLLR(:,:,iter-1) = sigmaLLR;
                end
            end
            report.SR{iter} = SR;
        end
        
        % -----------------------------------------------------------------
        % Step 1: Setup locally linear regression model
        % -----------------------------------------------------------------
        % Determine LLR filter coefficients and confidence weights for all
        % pairs of channels.
        for m = 1:numChannels
            for n = 1:numChannels
                if m ~= n                
                    % Determine the LLR filter coefficients for the current
                    % pair of channels
                    if isscalar(optimParams.llrHyperparameterWeights)
                        optimParams.llrHyperparameterWeights = optimParams.llrHyperparameterWeights .* ones(numChannels, numChannels);
                    end
                    % Determine adaptive confidence weights
                    if optimParams.adaptiveLLR
                        [A{m,n}, b{m,n}, R] = computeLLRFilterCoefficients(vectorToImage(SR{m}, model.magFactor * lrDim{m}), vectorToImage(SR{n}, model.magFactor * lrDim{m}), ...
                            optimParams.llrPatchRadius, optimParams.llrHyperparameterWeights(m,n), ...
                            optimParams.llrSamplingFactor, ...
                            vectorToImage(llrPrior.confidence{m,n}, model.magFactor * lrDim{m}));
                        % Select threshold to discriminate inliers and
                        % outliers in the LLR model.
                        sigmaLLR(m,n) = 1.4826 * weightedMedian( abs( R(:) - weightedMedian(R(:), llrPrior.confidence{m,n}) ), llrPrior.confidence{m,n} );
                        % Compute confidence weights with selected
                        % threshold
                        llrPrior.confidence{m,n} = computeLLRConfidenceWeights( imageToVector(R), sigmaLLR(m,n) );
                    else
                        % Use uniform weights.
                        [A{m,n}, b{m,n}, ~] = computeLLRFilterCoefficients(vectorToImage(SR{m}, model.magFactor * lrDim{m}), vectorToImage(SR{n}, model.magFactor * lrDim{m}), ...
                            optimParams.llrPatchRadius, optimParams.llrHyperparameterWeights(m,n), optimParams.llrSamplingFactor);
                    end
                    llrPrior.A{m,n} = imageToVector(A{m,n});
                    llrPrior.b{m,n} = imageToVector(b{m,n});
                end
            end
        end
        
        % -----------------------------------------------------------------
        % Step 2: Update multi-channel image
        % -----------------------------------------------------------------
        % Reshape cell arrays to parameter vector.
        numHRPixel = numel( SR{1} );
        SR_prev = zeros(numChannels * numHRPixel, 1);
        for m = 1:numChannels
            
            % Determine the low-resolution observation residual error for
            % the current channel.
            for frameIdx = 1:numFrames
                % Compute the residual error.
                I = LRImages{m};
                LR{m,frameIdx} = imageToVector(I(:,:,frameIdx));
                if size(W, 1) > 1
                    residualError{frameIdx} = getResidual(SR{m}, LR{m,frameIdx}, W{m,frameIdx}, model.photometricParams);
                else
                    residualError{frameIdx} = getResidual(SR{m}, LR{m,frameIdx}, W{frameIdx}, model.photometricParams);
                end
            end
            % Estimate the confidence weights for the current channel based
            % on the residual error and the Huber weighting function.
            observationWeightingFunction = estimateHuberObservationWeights;
            [model.confidence(m,:), sigmaNoise(m)] = observationWeightingFunction.function(residualError, model.confidence(m,:));
            
            % Determine the residual of the regularization term
            % corresponding to the intra-channel prior.
            %[~, transformedImage] =  model.imagePrior.function(SR{m}, model.imagePrior.parameters{1:end-1});
            [~, transformedImage] = optimParams.intraChannelPrior{m}.function(SR{m}, optimParams.intraChannelPrior{m}.parameters{1:end-1});
            % Filtering of the sparsity transformed image.
            transformedImageFiltered = [];
            if iscell(transformedImage)
                for l = 1:size(transformedImage,1)
                    for n = 1:size(transformedImage,2)
                        if ~isempty(transformedImage{l,n})
                            z = vectorToImage( transformedImage{l,n}, model.magFactor * size(LRImages{m}(:,:,1)));
                            transformedImageFiltered = [transformedImageFiltered; imageToVector( medfilt2(z, [3 3]) )];
                        end
                    end
                end
            else
                transformedImageFiltered = imageToVector( medfilt2(transformedImage, [3 3]) );
            end
            % Estimate the confidence weights of the intra-channel prior.
            if ~exist('btvWeights', 'var')
                % Initialize weights at the first iteration.
                btvWeights = ones(size(transformedImageFiltered));
            end
            if isscalar(optimParams.sparsityParameter)
                optimParams.sparsityParameter = optimParams.sparsityParameter * ones(numChannels, 1);
            end
            priorWeightingFunction = estimateWBTVPriorWeights;
            [btvWeights, sigmaBTV(m)] = priorWeightingFunction.function(transformedImageFiltered, btvWeights);
            % Update the parameters of the intra-channel prior.
            optimParams.intraChannelPrior{m}.parameters{1} = model.magFactor * size(I(:,:,1));
            optimParams.intraChannelPrior{m}.parameters{end} = btvWeights;
            
            % Capture estimate from the previous iteration.
            SR_prev(( (m-1)*numHRPixel + 1):(m*numHRPixel) ) = SR{m};
        end
        
        % Perform SCG iterations to update the multi-channel image.
        if isscalar(optimParams.interChannelWeights)
            optimParams.interChannelWeights = optimParams.interChannelWeights .* ones(numChannels, numChannels);
        end
        SR_new = scg(@mapfunc, SR_prev', ...    % Set objective function and initial guess
            scgOptions, @mapfunc_grad, ...      % Set optimization parameters and gradient of the objective funciton
            numChannels, numFrames, LR, W, Wt, model, llrPrior, optimParams);   % Set additional paraemters passed to the objective function
        
        % Check for convergence
        deltaSR = SR_new(:) - SR_prev(:);
        if max( abs(deltaSR) ) < optimParams.terminationTol
            isConverged = true;
        end
            
        % Reshape parameter vector into cell array.
        SR = mat2cell(SR_new', size(W{1}, 2) * ones(1, numChannels), 1);
        
        iter = iter + 1;
    end
    
    % Reshape parameter vector to a 2D image.
    for channelIdx = 1:numChannels
        SR{channelIdx} = vectorToImage(SR{channelIdx}, model.magFactor * lrDim{channelIdx});
    end
    
function scgOptions = setupSCGOptions(optimParams)

    scgOptions = zeros(1,18);
    scgOptions(2) = optimParams.terminationTol;
    scgOptions(3) = optimParams.terminationTol;
    scgOptions(10) = optimParams.maxSCGIter;
    scgOptions(14) = optimParams.maxSCGIter;

function f = mapfunc(SR, numChannels, numFrames, LR, W, ~, model, llrPrior, optimParams)
        
    % Convert parameter vectors into cell array for further computations.
    SR = mat2cell(SR', size(W{1,1}, 2) * ones(1,numChannels), 1);
    
    % Compute the data fidelity term channel-wise.
    fData = 0;
    for m = 1:numChannels 
        for frameIdx = 1:numFrames
            if size(W,1) > 1
                fData = fData + sum(model.confidence{m, frameIdx} .* (LR{m,frameIdx} - W{m,frameIdx} * SR{m}).^2);      
            else
                fData = fData + sum(model.confidence{m, frameIdx} .* (LR{m,frameIdx} - W{frameIdx} * SR{m}).^2); 
            end
        end     
    end
    
    % Evaluate intra-channel prior for each channel.
    fIntra = 0;
    for m = 1:numChannels
        fIntra = fIntra + optimParams.intraChannelPrior{m}.weight * optimParams.intraChannelPrior{m}.function(SR{m}, optimParams.intraChannelPrior{m}.parameters{1:end});
    end
    
    % Evaluate inter-channel prior for all pairs of channels.
    fInter = 0;
    for m = 1:numChannels
        for n = 1:numChannels
            if m ~= n
                fInter = fInter + optimParams.interChannelWeights(m,n) * sum( llrPrior.confidence{m,n} .* (llrPrior.A{m,n} .* SR{m} + llrPrior.b{m,n} - SR{n}).^2 );
            end
        end
    end
        
    % Calculate objective function.
    f = fData + fIntra + fInter;
                
function grad = mapfunc_grad(SR, numChannels, numFrames, LR, W, Wt, model, llrPrior, optimParams)
        
    % Convert parameter vectors into cell array for further computations.
    SR = mat2cell(SR', size(W{1,1}, 2) * ones(1,numChannels), 1);
    
    % Calculate gradient of the data fidelity term w.r.t. the individual
    % channels.
    numHRPixel = numel( SR{1} );
    gData = zeros(numChannels*numHRPixel, 1);
    for m = 1:numChannels
        for frameIdx = 1:numFrames
            if size(W,1) > 1
                gData( ((m-1)*numHRPixel + 1):(m*numHRPixel) ) = gData( ((m-1)*numHRPixel + 1):(m*numHRPixel) ) - 2 * Wt{m, frameIdx} * (model.confidence{m, frameIdx} .* (LR{m,frameIdx} - W{m,frameIdx}*SR{m}));
            else
                gData( ((m-1)*numHRPixel + 1):(m*numHRPixel) ) = gData( ((m-1)*numHRPixel + 1):(m*numHRPixel) ) - 2 * Wt{frameIdx} * (model.confidence{m, frameIdx} .* (LR{m,frameIdx} - W{frameIdx}*SR{m}));
            end
        end
    end
    
    % Calculate gradient of the intra-channel prior.
    gIntra = zeros(numChannels*numHRPixel, 1);
    for m = 1:numChannels
        gIntra( ((m-1)*numHRPixel + 1):(m*numHRPixel) ) = optimParams.intraChannelPrior{m}.weight * optimParams.intraChannelPrior{m}.gradient(SR{m}, optimParams.intraChannelPrior{m}.parameters{1:end});
    end
    
    % Calculate gradient of the inter-channel prior for all pairs of
    % channels.
    gInter = zeros(numChannels*numHRPixel, 1);
    for i = 1:numChannels
        g1 = 0;
        for n = 1:numChannels
            if n ~= i
                g1 = g1 + optimParams.interChannelWeights(i,n) * llrPrior.confidence{i,n} .* ( 2*llrPrior.A{i,n} .* (llrPrior.A{i,n} .* SR{i} + llrPrior.b{i,n} - SR{n}) );
            end
        end  
        g2 = 0;
        for m = 1:numChannels
            if m ~= i
                g2 = g2 - optimParams.interChannelWeights(m,i) * llrPrior.confidence{m,i} .* ( 2*(llrPrior.A{m,i} .* SR{m} + llrPrior.b{m,i} - SR{i}) );
            end
        end      
        gInter( ((i-1)*numHRPixel + 1):(i*numHRPixel) ) = g1 + g2;
    end
      
    % Compute overall gradient of the energy function.
    grad = gData + gIntra + gInter;
    grad = grad';   