% This demonstrates the application of our guided super-resolution
% algorithm applied to hybrid range imaging. Super-resolution is used to
% enhance the resolution of photometric and rage data acquired with
% Microsoft's Kinect. For details of this approach please refer to

% Florin C. Ghesu, Thomas Khler, Sven Haase and Joachim Hornegger, "Guided
% Image Super-Resolution: A New Technique for Photogeometric
% Super-Resolution in Hybrid 3-D Range Imaging", GCPR 2014
 
%% Setup all paths    
addpath('../');
setupSRToolbox;

%% Read example image data
% Load range and photometric data. Select a region of interest for
% super-resolution (set ROI = [] to select the whole image).
ROI = [100, 260, 300, 300];
I_range = readRangeSequence('data/Range', '*.txt', ROI);    % Stored as ASCII
I_photo = readImageSequence('data/RGB', '*.png', ROI);      % Stored as PNG

% Use grayscale converted RGB images.
numFrames = size(I_range, 3);
J = zeros(size(I_photo,1), size(I_photo,2), numFrames);
for k = 1:numFrames
    J(:,:,k) = im2double( rgb2gray(I_photo(:,:,:,k)) );
end
I_photo = J;

% Normalize range data to [0, 1] for super-resolution reconstruction (can
% be-denormalized for further processing).
minI = min(I_range(:));
maxI = max(I_range(:));
I_range = (I_range - minI) ./ (maxI - minI);

%% Motion estimation

% Set parameters for optical flow.
ofl_param = [0.025, ...  % Regularization weight
             0.5, ...    % Downsample ratio
             20, ...     % Width of image pyramid coarsest level
             7, ...      % Number of outer fixed point iterations
             1, ...      % Number of inner fixed point iterations
             20];        % Number of SOR iterations

% Apply optical flow for motion estimation. Each frame is warped to a fixed
% reference frame (= central frame of the input image sequence).
refFrameIdx = 1;
for k = 1 : numFrames
    if (k == refFrameIdx)
        % Set displacements to zero for the reference frame.
        motionParams{k}.vx = zeros(size(I_photo(:,:,1)));
        motionParams{k}.vy = zeros(size(I_photo(:,:,1)));
    else
        % Estimate subpixel displacements FROM current frame TO the
        % reference frame using coarse-to-fine optical flow.
        [motionParams{k}.vx, ...    % x-component of displacement vector
         motionParams{k}.vy] ...    % y-component of displacement vector
            = Coarse2FineTwoFrames(I_photo(:,:,k), I_photo(:,:,refFrameIdx), ofl_param);
    end
end
 
%% Single-sensor super-resolution for each modality separately

% Set model parameters for photometric data.
modelPhoto = SRModel;
modelPhoto.magFactor = 2;     % 2x magnification
modelPhoto.psfWidth = 0.4;    % Radius of Gaussian PSF
modelPhoto.motionParams = motionParams;
modelPhoto.imagePrior = SRPrior('function', @btvPrior, 'gradient', @btvPrior_grad, ...
                                'weight', 0.005, ...                                            % Smoothness regularization weight for photometric data (1e-4 ... 1e-3)
                                'parameters', {modelPhoto.magFactor * size(I_photo(:,:,1))});
% Set parameters for non-linear optimization.
solverParams = SRSolverParams('maxIter', 20, 'maxFunEvals', 20);
% Super-resolve photometric data. We use MAP-based super-resolution
% reconstruction soley on the photometric data.
SR_photo = superresolve(I_photo, modelPhoto, 'map', solverParams);

% Set model parameters for range data.
modelRange = SRModel;
modelRange.magFactor = 2;     % 2x magnification
modelRange.psfWidth = 0.6;    % Radius of Gaussian PSF
modelRange.motionParams = motionParams;
modelRange.imagePrior = SRPrior('function', @btvPrior, 'gradient', @btvPrior_grad, ...
                                'weight', 0.002, ...                                            % Smoothness regularization weight for range data (1e-4 ... 1e-3)
                                'parameters', {modelRange.magFactor * size(I_photo(:,:,1))});
% Assign zero confidence to pixels selected as invalid (indicated by range
% value 0).
modelRange.confidence = imageToVector(I_range) > 0;
% Super-resolve range data. We use the multi-sensor super-resolution to
% derive the motion estimate for range images from the complementary 
% photometric data.
[SR_range, modelRange] = superresolve(I_range, modelRange, 'multisens', solverParams);

%% Guided super-resolution

% Set parameters for smoothness regularization (based on BTV
% regularization).
modelPhoto.imagePrior = SRPrior('function', @btvPrior, 'gradient', @btvPrior_grad, ...
                                'weight', 0.002, ...                                            % Smoothness regularization weight for photometric data (1e-4 ... 1e-3)
                                'parameters', {modelPhoto.magFactor * size(I_photo(:,:,1))});
% We use the single-sensor super-resolution as initial guess (omitt this
% parameter to select the initial guess automatically).
modelPhoto.SR = SR_photo;

modelRange.imagePrior = SRPrior('function', @btvPrior, 'gradient', @btvPrior_grad, ...
                                'weight', 0.0001, ...                                           % Smoothness regularization weight for range data (0 ... 1e-4)
                                'parameters', {modelRange.magFactor * size(I_photo(:,:,1))});
% We use the single-sensor super-resolution as initial guess (omitt this
% parameter to select the initial guess automatically)
modelRange.SR = SR_range;
% Assign zero confidence to pixels selected as invalid (indicated by range
% value 0).
modelRange.confidence = imageToVector(I_range) > 0;

% Set parameters for non-linear optimization.
solverParams = SRSolverParams('maxIter', 5, ...         % No. of inner iterations (SCG iterations)
                              'maxFunEvals', 5, ...
                              'maxIrlsIter', 10);       % No. of IRLS iterations

% Set guided filter parameters for interdependence regularization.
gf_eps = 1e-4;      % Guided filter regularization weight (1e-6 ... 1e-2)
gf_radius = 1;      % Guided filter radius (1 ... 3)
% Set weight for interdependence regularization.
gf_lambda = 0.5;    % (0.5 ... 2, set to 0 to disable guidance)
% Apply guided super-resolution to super-resolve range and photometric
[SR_guided_range, SR_guided_photo] = superresolve(I_range, modelRange, ...                        % Range data used as input
                                                  'guidedsr', ...
                                                  I_photo, modelPhoto, ...                        % Photometric data used as guidance                     
                                                  @irlsWeightFunctionHuber, ...                   % Handle to weight function used for IRLS optimization
                                                  gf_radius, gf_eps, gf_lambda, solverParams);
                                          
%% Visualize photometric data and range images
figure; imshow([imadjust(imresize(I_range(:,:,refFrameIdx), 2, 'method', 'nearest')) imadjust(SR_range) imadjust(SR_guided_range)]);
title('Range data: low-resolution, single-sensor super-resolution, guided super-resolution');
figure; imshow([imadjust(imresize(I_photo(:,:,refFrameIdx), 2, 'method', 'nearest')) imadjust(SR_photo) imadjust(SR_guided_photo)]);
title('Photometric data: low-resolution, single-sensor super-resolution, guided super-resolution');