%% Setup all paths    
addpath('../');
setupSRToolbox;

%% Load low-resolution input data
% Chose the number of frames
numFrames = 15;
I_range = readRangeSequence('Data/baselineSequence/Range', '*.txt');
numFrames = min(numFrames, size(I_range,3));
I_range = I_range(:,:,1:numFrames);
I_RGB = readImageSequence('Data/baselineSequence/RGB', '*.png');
I_RGB = I_RGB(:,:,:,1:numFrames);

% Data normalization
I_range = (I_range - min(I_range(:))) ./ (max(I_range(:)) - min(I_range(:)));

%% Motion estimation
% Use optical flow to estimate subpixel displacements.
numFrames = size(I_range, 3);
ofl_param = [0.025, ...  % Regularization weight
             0.5, ...    % Downsample ratio
             20, ...     % Width of image pyramid coarsest level
             7, ...      % Number of outer fixed point iterations
             1, ...      % Number of inner fixed point iterations
             20];        % Number of SOR iterations
 
 % Choose central frame as reference.
 refFrameIdx = round((numFrames+1)/2); 
 for k = 1:numFrames
    if k == refFrameIdx
        % Set displacements to zero for the reference frame.
        motionParams{k}.vx = zeros(size(I_RGB(:,:,1)));
        motionParams{k}.vy = zeros(size(I_RGB(:,:,1)));
        I_warped(:,:,:,k) = I_RGB(:,:,:,k);
    else
        % Estimate subpixel displacements FROM current frame TO the
        % reference frame using coarse-to-fine optical flow.
        [motionParams{k}.vx, motionParams{k}.vy, ~] = Coarse2FineTwoFrames(im2double(I_RGB(:,:,:,k)), im2double(I_RGB(:,:,:,refFrameIdx)), ofl_param);
        [~, ~, I_warped(:,:,:,k)] = Coarse2FineTwoFrames(im2double(I_RGB(:,:,:,refFrameIdx)), im2double(I_RGB(:,:,:,k)), ofl_param); %#ok<*AGROW>
    end
end


%% Super-resolution reconstruction
% Setup model parameters
model = SRModel;
% Set the desired magnification factor.
model.magFactor = 4;
model.psfWidth = 0.5;
% Set motion parameters used for super-resolution.
model.motionParams = motionParams;

% Setup regularization using Huber prior
lambda = 0.2;    % Regularization weight
alpha = 5e-3;   % Huber threshold
D = getLaplacianMatrix(model.magFactor * size(I_range(:,:,1)));
model.imagePrior = SRPrior('function', @huberPrior, 'gradient', @huberPrior_grad, 'weight', lambda, 'parameters', {D, alpha});

% Error model for low-resolution images used for image reconstruction. We
% use least square estimation in this example.
model.errorModel = 'l2NormErrorModel';

% Set initial guess for super-resolved image. For this example, the bicubic
% upsampled reference frame is used as initial guess.
I_bc = imresize(I_range(:,:,refFrameIdx), model.magFactor);
model.SR = I_bc;

% Multi-sensor super-resolution based on unweighted L2 minimization
solverParams = SRSolverParams('maxFunEvals', 25, 'maxIter', 25, 'tolX', 1e-3, 'tolF', 1e-3, 'verbose', true);
[I_msr, model] = superresolve(I_range, model, 'multisens', solverParams);

% Adaptive multi-sensor super-resolution using spatially adaptive Huber
% regularization.
amsrPatchSize = 12;
amsrContrastFactor = 0.0250;
% Determine spatially adaptive regularization weights using RGB data and
% super-resolved range data obtained with unweighted Huber prior.
amsrWeights = getSpatiallyAdaptiveRegularizationWeights(I_msr, median(I_warped, 4), amsrPatchSize, amsrContrastFactor);
model.imagePrior = SRPrior('function', @huberPrior, 'gradient', @huberPrior_grad, 'weight', lambda, 'parameters', {D, alpha, amsrWeights});
model.SR = I_msr;
I_amsr = superresolve(I_range, model, 'multisens', solverParams);

figure; imshow(I_range(:,:,1));
title('Range image');
figure; imshow(I_RGB(:,:,:,1));
title('Photometric data (RGB image)');
figure; imshow([I_bc I_msr I_amsr]);
title('From left to right: bicubic upsampling, multi-sensor super-resolution, adaptive multi-sensor super-resolution');